/* 
   ORSA - Orbit Reconstruction, Simulation and Analysis
   Copyright (C) 2002-2003 Pasquale Tricarico
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef _ORSA_CONFIG_H_
#define _ORSA_CONFIG_H_

#include <string>
#include <map>

using namespace std;

namespace orsa {
  
  class OrsaConfigFile;
  
  template <class T> class ConfigItem {
  public:
    // ConfigItem(const T v, const string l, const string d, const string t) : value(v), label(l), description(d), tag(t) { };
    ConfigItem(const string t) : tag(t) { };
    ConfigItem(const T v, const string t) : value(v), tag(t) { };
    
  public:
    T GetValue() const       { return value; }
    void SetValue(const T v) { value = v; }
    
  private:
    T value;
    
  public:
    //! label is a human_readable, constant length label, which appears in interactive programs; descr is a description of the option; tag is the tag in the configuraiton file
    // const string label, description, tag;
    const string tag;
  };
  
  enum ConfigEnum {
    JPL_EPHEM_FILE,
    JPL_DASTCOM_NUM,
    JPL_DASTCOM_UNNUM,
    JPL_DASTCOM_COMET,
    LOWELL_ASTORB,
    MPC_MPCORB,
    MPC_COMET,
    MPC_NEA,
    MPC_DAILY,
    MPC_DISTANT,
    MPC_PHA,
    MPC_UNUSUALS,
    ASTDYS_ALLNUM_CAT,
    ASTDYS_ALLNUM_CTC,
    ASTDYS_ALLNUM_CTM,
    NEODYS_CAT,
    NEODYS_CTC,
    OBSCODE,
    // aliases for backward compatibility, to be removed in future
    ASTORB_DAT=LOWELL_ASTORB,
    MPCORB_DAT=MPC_MPCORB,
    JPLEPH=JPL_EPHEM_FILE
  };
  
  string Label(const ConfigEnum);
  string Description(const ConfigEnum);
  
  class Config {
  public:
    Config();
    
  public:
    map < ConfigEnum, ConfigItem<string>* > paths;
    
  };
  
} // namespace orsa

#endif // _ORSA_FILE_H_
