/* 
   ORSA - Orbit Reconstruction, Simulation and Analysis
   Copyright (C) 2002-2003 Pasquale Tricarico
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef _ORSA_FILE_JPL_H_
#define _ORSA_FILE_JPL_H_

#include <string>
#include <map>

namespace orsa {
  
  enum JPL_planets {MERCURY=1,VENUS=2,EARTH=3,MARS=4,JUPITER=5,SATURN=6,URANUS=7,NEPTUNE=8,PLUTO=9,MOON=10,SUN=11,SOLAR_SYSTEM_BARYCENTER=12,EARTH_MOON_BARYCENTER=13,NUTATIONS=14,LIBRATIONS=15,EARTH_AND_MOON=1000};
  // note: EARTH_AND_MOON is only an useful definition, not used in orsa files or by JPL...
  
  //! JPL ephem file  
  class JPLFile {
  public:
    JPLFile(string);
    ~JPLFile();
    
  public:
    void GetEph(const UniverseTypeAwareTime &date, JPL_planets target, JPL_planets center, Vector &position, Vector &velocity);       
    inline void GetEph(const UniverseTypeAwareTime &date, JPL_planets target, Vector &position, Vector &velocity) {
      GetEph(date, target, default_ephem_center, position, velocity);       
    }
    
    UniverseTypeAwareTime EphemStart() { return ephem_start; };
    UniverseTypeAwareTime EphemEnd()   { return ephem_end;   };
    
    double GetMass(JPL_planets planet);
    
    double GetAU_MKS();
    double GetMSun_MKS();
    double GetMJupiter_MKS();
    double GetMEarth_MKS();
    double GetMMoon_MKS();
    double GetC_MKS(); // speed of light
    double GetREarth_MKS();
    double GetRMoon_MKS();
    
  public:
    double GetTag(string);
    
  private:
    map <string,double> map_tag;
    
  private:
    static const JPL_planets default_ephem_center;
    
  private:
    void   *jpl_database;
    char    nam[65536][6];
    double  val[65536]; 
    double  xv[6];
    int     calc_velocity; // integer flag; if nonzero, velocities will be computed, otherwise not. 
    // 
    UniverseTypeAwareTime ephem_start, ephem_end;
  };  
  
  bool is_genuine_planet(const JPL_planets p);
  
  string JPL_planet_name(const JPL_planets p);
  
  // used ONLY for 3D graphical purposes
  double radius(const JPL_planets p);
  
} // namespace orsa

#endif // _ORSA_FILE_JPL_H_
