/* 
   ORSA - Orbit Reconstruction, Simulation and Analysis
   Copyright (C) 2002-2003 Pasquale Tricarico
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef _ORSA_FRAME_H_
#define _ORSA_FRAME_H_

#include <vector>
#include <list>

#include "orsa_body.h"
#include "orsa_common.h"
#include "orsa_file_jpl.h"

using namespace std;

namespace orsa {
  
  class Frame : public vector<Body>, public UniverseTypeAwareTime {
  public:
    Frame();
    Frame(const Frame &f);
    
    // used to sort frames for increasing time 
    bool operator < (const Frame &f) const;
    
  public:
    Vector CenterOfMass();
    Vector Baricenter();
    Vector AngularMomentum(const Vector&);
    Vector BaricentricAngularMomentum();
  };
  
  void print(const Frame&);
  
  double KineticEnergy(const vector<Body>&);
  
  Vector CenterOfMass(const vector<Body>&);
  Vector Baricenter(const vector<Body>&);
  Vector AngularMomentum(const vector<Body>&, const Vector&);
  Vector BaricentricAngularMomentum(const vector<Body>&);
  
  // 
  
  void SetupSolarSystem(Frame&, const list<JPL_planets>&, const UniverseTypeAwareTime&);
  
} // namespace orsa

#endif // _ORSA_FRAME_H_
