/* 
   ORSA - Orbit Reconstruction, Simulation and Analysis
   Copyright (C) 2002-2003 Pasquale Tricarico
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef _ORSA_ORBIT_GSL_H_
#define _ORSA_ORBIT_GSL_H_

#include "orsa_orbit.h"

namespace orsa {
  
  class PreliminaryOrbit : public OrbitWithEpoch {
  public:
    double GetRMS() const { return rms; };
    
  public:	
    void ComputeRMS(const vector<Observation>&);
    
  public:
    // used to sort 
    inline bool operator < (const PreliminaryOrbit &orbit) const {
      return (rms < orbit.rms);
    }
    
  private:
    double rms;
  };
  
  // Minimal Orbit Intersection Distance between two orbits
  double MOID(const Orbit&, const Orbit&, Vector&, Vector&);
  
  // Minimal Orbit Intersection Distance between two orbits with different reference bodies  
  double MOID2RB(const Vector&, const Vector&, const Orbit&, const Orbit&, Vector&, Vector&);
  
  //! General interface to the Orbit computation from a set of observations.
  OrbitWithEpoch Compute(const vector<Observation>&);
  void Compute_Gauss(const vector<Observation>&, vector<PreliminaryOrbit>&);
  void Compute_TestMethod(const vector<Observation>&, vector<PreliminaryOrbit>&);
  
  class OrbitWithCovarianceMatrixGSL : public OrbitWithEpoch {
  public:
    OrbitWithCovarianceMatrixGSL(Orbit&,double**,CovarianceMatrixElements);
    
  private:
    double covm[6][6];
    CovarianceMatrixElements cov_base;
  public:
    void generate(vector<Orbit>&,const int);
  };
  
}

#endif // _ORSA_ORBIT_GSL_H_
